import * as section from './section.js';

// Fires a LB+ drop zone event.
function dropZoneListener(dropZone) {
  dropZone.ondragover = e => {
    e.preventDefault();
  };
  dropZone.ondragenter = e => {
    e.target.classList.add('drop-zone-hover');
  };
  dropZone.ondragleave = e => {
    e.target.classList.remove('drop-zone-hover');
  };
  dropZone.ondrop = e => {
    e.preventDefault();
    const event = new CustomEvent('element-dropped', {
      bubbles: true,
      cancelable: true,
      detail: {
        dropEvent: e,
        ...JSON.parse(e.dataTransfer.getData('text/json')),
      },
    });

    document.dispatchEvent(event);
    clearDropZones();
  };
  return dropZone;
}

export const clearDropZones = () => {
  document.querySelectorAll('.drop-zone').forEach(section => {
    section.remove();
  });
}

// Adds drop zones around sections.
export const addSectionDropZones = (draggedSection, message = Drupal.t('Move here')) => {
  const sections = section.getSections();
  const dropZone = document.createElement('div');
  dropZone.classList.add('drop-zone', 'section')
  dropZone.innerText = message;

  if (sections.length > 0) {
    // Add drop zones above each section.
    sections.forEach(section => {
      if (
        (
          // A block is being dragged.
          (draggedSection === null) ||
          // The previous element is not the section being dragged.
          (section !== draggedSection && section.previousElementSibling !== draggedSection)
        ) &&
        // There's not already a drop zone here.
        !section.previousElementSibling?.classList.contains('drop-zone') &&
        // The section is scrolled in to view.
        isElementVisible(section)
      ) {
        let sectionDropZone = dropZone.cloneNode(true);
        sectionDropZone.setAttribute('drop-zone-type', 'section');
        section.parentNode.insertBefore(dropZoneListener(sectionDropZone), section);
      }
    });
    // Add a drop zone below the last section.
    const lastSection = sections[sections.length - 1];
    if (
      !lastSection.nextElementSibling?.classList.contains('drop-zone') &&
      isElementVisible(lastSection)
    ) {
      let sectionDropZone = dropZone.cloneNode(true);
      sectionDropZone.setAttribute('drop-zone-type', 'section');
      lastSection.parentNode.insertBefore(dropZoneListener(sectionDropZone), lastSection.nextSibling);
    }
  }
  else {
    // There are no sections so add 1 section drop zone.
    if (document.getElementsByClassName('drop-zone').length === 0) {
      let sectionDropZone = dropZone.cloneNode(true);
      sectionDropZone.setAttribute('drop-zone-type', 'section');
      document.querySelector('.layout-builder.active').appendChild(dropZoneListener(sectionDropZone));
    }
  }
}

// Adds drop zones at the top and bottom of each region.
export const addRegionDropZones = () => {
  const dropZone = document.createElement('div');
  dropZone.classList.add('drop-zone', 'layout')
  dropZone.innerText = Drupal.t('Add to region');
  let columns = document.querySelectorAll('.layout-builder.active .lb-plus-section .js-layout-builder-region')
  columns.forEach(column => {
    let regionDropZone = dropZone.cloneNode(true);
    regionDropZone.setAttribute('drop-zone-type', 'region');
    // Add drop zones before each region.
    if (
      !column.firstElementChild?.classList.contains('drop-zone') &&
      isElementVisible(column)
    ) {
      regionDropZone.setAttribute('position', 'above');
      column.insertBefore(dropZoneListener(regionDropZone), column.firstChild);
    }
    // Add drop zones after each region.
    if (
      !column.lastElementChild?.classList.contains('drop-zone') &&
      isElementVisible(column)
    ) {
      regionDropZone.setAttribute('position', 'below');
      column.appendChild(dropZoneListener(regionDropZone));
    }
  });
}

function isElementVisible(section) {
  const rect = section.getBoundingClientRect();
  return (
    (
      // Top of div is below the viewport
      rect.top >= 0 &&
      rect.top < (window.innerHeight || document.body.clientHeight)
    ) ||
    (
      // Bottom of div is above the bottom of the viewport.
      rect.bottom <= (window.innerHeight || document.body.clientHeight) &&
      // Bottom of div is not above the screen.
      rect.bottom >= 0
    )
  );
}
