export default function ($ = jQuery, Drupal, once, dropZones) {

  // Sort handle icon.
  Drupal.behaviors.LBPlusDraggableSectionHandles = {
    attach: (context, settings) => {
      once('draggableSectionHandles', '.lb-plus-section-admin .sort', context).forEach(sortHandle => {
        sortHandle.onmousedown = e => {
          // The drag handle was clicked, set the section to draggable.
          let draggedSection = e.target.closest('.lb-plus-section');
          draggedSection.setAttribute('draggable', 'true');
        }
      });
    }
  };

  // Draggable/sortable sections.
  Drupal.behaviors.LBPlusDraggableSections = {
    attach: (context, settings) => {
      once('draggableSections', '.lb-plus-section', context).forEach(draggableSection => {
        draggableSection.ondrag = e => {
          if (e.target.classList.contains('layout-builder__section')) {
            dropZones.addSectionDropZones(e.target);
          }
        };
        draggableSection.ondragstart = e => {
          // Store the element ID for placing once the element has been dropped.
          e.dataTransfer.setData('text/json', JSON.stringify({
            id: e.target.id,
            type: 'section',
          }));
          let draggedSection = e.target;
          // Disable hovers on other sections.
          getSections().forEach(draggableSection => {
            if (draggableSection !== draggedSection) {
              draggableSection.classList.remove('hover');
            }
          });
        };
        draggableSection.ondragend = e => {
          dropZones.clearDropZones();
          // The sort icon set this section to draggable. Return it to not
          // draggable now that the section has been dropped so that text can be
          // selected etc.
          e.target.setAttribute('draggable', 'false');
          // Enable section hovers.
          getSections().forEach(draggableSection => {
            if (!draggableSection.classList.contains('hover')) {
              draggableSection.classList.add('hover');
            }
          });
        };
      });
    },
  };

  // A section was dropped.
  document.addEventListener('element-dropped', e => {
    if (e.detail.type !== 'section' && e.detail.type !== 'new_section') {
      return;
    }

    let ajaxConfig = {
      type: 'POST',
      dataType: 'text',
      progress: {
        type: 'fullscreen',
        message: Drupal.t('Saving section...')
      },
    };

    // A section was moved.
    if (e.detail.type === 'section') {
      // Get the original order of the sections in case this operation fails.
      let originalSectionsOrder = [];
      getSections().forEach(section => {
        originalSectionsOrder.push(section.id);
      });
      // Move the section.
      const draggedSection = document.getElementById(e.detail.id);
      const dropZone = e.detail.dropEvent.target;
      dropZone.parentElement.insertBefore(draggedSection, dropZone);
      // Get the new order of the sections.
      let sectionsOrder = [];
      getSections().forEach(section => {
        sectionsOrder.push(section.id);
      });

      // Save the section order.
      ajaxConfig.url = '/lb-plus/move-section/' + drupalSettings['LB+'].sectionStorageType + '/' + drupalSettings['LB+'].sectionStorage;
      ajaxConfig.submit = {sections_order: sectionsOrder};
      ajaxConfig.error = (xhr, status, error) => {
        originalSectionsOrder.forEach(sectionId => {
          // Updating failed, put the sections back how they were before.
          const section = document.getElementById(sectionId);
          section.parentElement.appendChild(section);
        });
        sectionAjaxError('<span id="update-error">Unable to save the section order.</span>', {type: 'error'});
        console.log(error);
      };
    }

    // A new section was placed.
    if (e.detail.type === 'new_section') {
      const dropZone = e.detail.dropEvent.target;
      const nextSection = dropZone.nextElementSibling;
      let nextSectionId = 'last';
      if (nextSection && nextSection.classList.contains('lb-plus-section')) {
        nextSectionId = nextSection.id;
      }

      // Place an empty section.
      ajaxConfig.url = '/lb-plus/add-empty-section/' + drupalSettings['LB+'].sectionStorageType + '/' + drupalSettings['LB+'].sectionStorage;
      ajaxConfig.submit = {next_section: nextSectionId};
      ajaxConfig.error = (xhr, status, error) => {
        sectionAjaxError('<span id="update-error">Unable to save the new section.</span>');
      };
    }
    if (drupalSettings['LB+'].isLayoutBlock) {
      ajaxConfig.url = ajaxConfig.url + '/' + drupalSettings['LB+'].nestedStoragePath;
    }
    let ajax = Drupal.ajax(ajaxConfig);
    ajax.execute();
  });

}

// Removes the ajax spinner and displays an error message.
function sectionAjaxError(message) {
  // Remove the AJAX progress spinner.
  document.querySelectorAll('.ajax-progress').forEach(progress => {
    progress.remove();
  });
  // Display a message.
  const messages = new Drupal.Message();
  messages.add(message, {type: 'error'});
  document.getElementById('update-error').scrollIntoView({behavior: 'smooth', block: 'start'});
}

export const getSections = () => {
  return document.querySelectorAll('.layout-builder.active .lb-plus-section');
};

// Scroll up and down when items are dragged to the top and bottom of the page.
jQuery(document).ready(() => {
  jQuery().dndPageScroll({
    delay: 10,
    height: 50,
  });
});

// Clicks the configure link on a section.
jQuery.fn.LBPlusConfigureSection = (uuid) => {
  jQuery('#' + uuid).find('.configure-link').click();
};
