<?php

namespace Drupal\lb_plus\Controller;

use Drupal\Component\Uuid\Php;
use Drupal\Core\Render\Element;
use Drupal\layout_builder\Section;
use Drupal\lb_plus\LbPlusRebuildTrait;
use Drupal\lb_plus\LbPlusSettingsTrait;
use Drupal\lb_plus\SectionStorageHandler;
use Drupal\lb_plus\Event\PlaceBlockEvent;
use Drupal\Core\Controller\ControllerBase;
use Drupal\layout_builder\SectionComponent;
use Drupal\Core\Layout\LayoutPluginManager;
use Drupal\Core\Block\BlockManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\layout_builder\Context\LayoutBuilderContextTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Returns responses for Layout Builder Plus routes.
 */
class DropZones extends ControllerBase {

  use ContextAwarePluginAssignmentTrait;
  use LayoutBuilderContextTrait;
  use LbPlusSettingsTrait;
  use LbPlusRebuildTrait;

  protected Php $uuid;
  protected LayoutPluginManager $layoutManager;
  protected BlockManagerInterface $blockManager;
  protected EventDispatcherInterface $eventDispatcher;
  protected SectionStorageHandler $sectionStorageHandler;
  protected LayoutTempstoreRepositoryInterface $layoutTempstoreRepository;

  public function __construct(LayoutTempstoreRepositoryInterface $tempstore_repository, SectionStorageHandler $section_storage_handler, LayoutPluginManager $layout_manager, BlockManagerInterface $block_manager, EventDispatcherInterface $event_dispatcher, Php $uuid) {
    $this->layoutTempstoreRepository = $tempstore_repository;
    $this->sectionStorageHandler = $section_storage_handler;
    $this->eventDispatcher = $event_dispatcher;
    $this->layoutManager = $layout_manager;
    $this->blockManager = $block_manager;
    $this->uuid = $uuid;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_builder.tempstore_repository'),
      $container->get('lb_plus.section_storage_handler'),
      $container->get('plugin.manager.core.layout'),
      $container->get('plugin.manager.block'),
      $container->get('event_dispatcher'),
      $container->get('uuid')
    );
  }

  /**
   * Move section.
   *
   * Stores the updated position after a section has been moved and reordered.
   */
  public function moveSection(Request $request, SectionStorageInterface $section_storage, string $nested_storage_path = NULL) {
    $current_section_storage = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path);
    // Get the new sort order from drop-zones.js.
    $sections_order = $request->get('sections_order');
    if (empty($sections_order)) {
      throw new \InvalidArgumentException('Missing section order.');
    }
    $sections_order = array_flip($sections_order);
    // Update section storage with the updated sort order.
    $sections = $current_section_storage->getSections();
    $current_section_storage->removeAllSections();
    foreach ($sections as $section) {
      $section_uuid = $section->getThirdPartySetting('lb_plus', 'uuid');
      if (is_numeric($sections_order[$section_uuid])) {
        $current_section_storage->insertSection($sections_order[$section_uuid], $section);
      }
    }
    $section_storage = $this->sectionStorageHandler->updateSectionStorage($section_storage, $nested_storage_path, $current_section_storage);

    return $this->rebuildLayout($section_storage, $nested_storage_path);
  }

  /**
   * Move section.
   *
   * Stores the updated position after a section has been moved and reordered.
   */
  public function addEmptySection(Request $request, SectionStorageInterface $section_storage, string $nested_storage_path = NULL) {
    $current_section_storage = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path);
    // Get the new next_section from drop-zones.js.
    $next_section = $request->get('next_section');

    // Place the a blank section.
    $this->getSection([
      'type' => 'section',
      'section' => $next_section,
    ], $current_section_storage);

    $section_storage = $this->sectionStorageHandler->updateSectionStorage($section_storage, $nested_storage_path, $current_section_storage);

    return $this->rebuildLayout($section_storage, $nested_storage_path);
  }

  /**
   * Place block.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param string|null $nested_storage_path
   *   The nested storage path.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The layout builder UI with the updated block placement.
   *
   * @throws \Drupal\Component\Plugin\Exception\ContextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function placeBlock(Request $request, SectionStorageInterface $section_storage, string $nested_storage_path = NULL) {
    $current_section_storage = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path);

    // Get the block information from block.js.
    $block_info = $request->get('place_block');
    if (empty($block_info)) {
      throw new \InvalidArgumentException('Missing block placement info.');
    }
    // Ensure we have a valid block plugin ID.
    $block_definitions = $this->blockManager->getDefinitions();
    if (empty($block_definitions[$block_info['plugin_id']])) {
      throw new \InvalidArgumentException('Invalid block_plugin_id');
    }

    // Ensure we have a valid destination type.
    if (!in_array($block_info['destination']['type'], ['region', 'section'])) {
      throw new \InvalidArgumentException('Invalid block_plugin_id');
    }

    $section = $this->getSection($block_info['destination'], $current_section_storage);
    if (is_null($section->getLayoutId())) {
      throw new \InvalidArgumentException('Please configure a default layout for this section.');
    }

    // Ensure we have a valid region.
    if ($block_info['destination']['type'] === 'region' && empty($this->layoutManager->getDefinition($section->getLayoutId())->getRegions()[$block_info['destination']['region']])) {
      throw new \InvalidArgumentException('Invalid region to place block.');
    }
    $region = $block_info['destination']['region'];

    // Figure out placement (delta).
    if ($block_info['destination']['type'] === 'region') {
      $components = $section->getComponentsByRegion($region);
      $region_delta = $block_info['destination']['position'] === 'below' ? count($components) : 0;
    }
    else {
      $region = $section->getDefaultRegion();
      $region_delta = 0;
    }

    // Create a block to place.
    $block_plugin = $this->blockManager->createInstance($block_info['plugin_id']);

    // Add context mapping to the configuration. Field blocks especially expect
    // context mapping.
    $configuration = $block_plugin->getConfiguration();
    $assigned_context_element = $this->addContextAssignmentElement($block_plugin, $this->getPopulatedContexts($current_section_storage));
    foreach (Element::children($assigned_context_element) as $key) {
      $configuration['context_mapping'][$key] = $assigned_context_element[$key]['#value'];
    }

    [$block_plugin_id, $bundle] = explode(':', $block_plugin->getPluginId());

    if ($block_plugin_id === 'inline_block') {
      // Create a block content entity with placeholder content.
      $block_content = $this->entityTypeManager()->getStorage('block_content')->create([
        'type' => $bundle,
        'reusable' => FALSE,
      ]);
      foreach ($block_content as $field) {
        if ($field->getFieldDefinition()->getFieldStorageDefinition()->isBaseField()) {
          continue;
        }
        if ($this->moduleHandler()->moduleExists('field_sample_value')) {
          \Drupal::service('field_sample_value.generator')->populateWithSampleValue($field);
        } else {
          $field->generateSampleItems();
        }
      }
      $configuration['block_serialized'] = serialize($block_content);
    }

    $configuration['label_display'] = 0;
    $block_plugin->setConfiguration($configuration);
    $this->eventDispatcher->dispatch(new PlaceBlockEvent($block_plugin), PlaceBlockEvent::class);

    // Add the new block to the section.
    $component = new SectionComponent($this->uuid->generate(), $region, ['id' => $block_info['plugin_id']]);
    $component->setConfiguration($block_plugin->getConfiguration());
    $section->insertComponent($region_delta, $component);

    $section_storage = $this->sectionStorageHandler->updateSectionStorage($section_storage, $nested_storage_path, $current_section_storage);

    return $this->rebuildLayout($section_storage, $nested_storage_path);
  }

  /**
   * Get section.
   *
   * Find the relevant section or create a new one for block placement.
   *
   * @param array $destination
   *   The block section destination.
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   *
   * @return \Drupal\layout_builder\Section
   *   The section to place the block.
   *
   * @throws \Drupal\Component\Plugin\Exception\ContextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getSection(array $destination, SectionStorageInterface $section_storage) {
    if ($destination['type'] === 'section' && $destination['section'] === 'last') {
      $delta = $section_storage->count();
    }
    else {
      // Find the section and delta.
      for ($delta = 0; $delta < $section_storage->count(); $delta++) {
        $section = $section_storage->getSection($delta);
        if ($section->getThirdPartySetting('lb_plus', 'uuid') === $destination['section']) {
          break;
        }
      }
    }

    // Insert a new section.
    if ($destination['type'] === 'section') {
      $layout_settings = $this->getLbPlusSetting($section_storage, 'default_section');
      $layout_plugin_id = $layout_settings['layout_plugin'];
      $section = new Section($layout_plugin_id, $layout_settings);
      $section_uuid = $this->uuid->generate();
      $section->setThirdPartySetting('lb_plus', 'uuid', $section_uuid);
      $section_storage->insertSection($delta, $section);
    }

    return $section;
  }

}
