<?php

namespace Drupal\lb_plus\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\lb_plus\LbPlusRebuildTrait;
use Drupal\lb_plus\SectionStorageHandler;
use Symfony\Component\HttpFoundation\Request;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\layout_builder\LayoutTempstoreRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;

/**
 * Defines a controller to move a block.
 *
 * @internal
 *   Controller classes are internal.
 */
class MoveBlockController implements ContainerInjectionInterface {

  use LbPlusRebuildTrait;

  protected LayoutTempstoreRepositoryInterface $layoutTempstoreRepository;
  protected SectionStorageHandler $sectionStorageHandler;

  public function __construct(LayoutTempstoreRepositoryInterface $layout_tempstore_repository, SectionStorageHandler $section_storage_handler) {
    $this->layoutTempstoreRepository = $layout_tempstore_repository;
    $this->sectionStorageHandler = $section_storage_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('layout_builder.tempstore_repository'),
      $container->get('lb_plus.section_storage_handler')
    );
  }

  /**
   * Moves a block to another region.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\layout_builder\SectionStorageInterface $section_storage
   *   The section storage.
   * @param string|null $nested_storage_path
   *   The nested storage path.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The rebuilt LB.
   *
   * @throws \Drupal\Component\Plugin\Exception\ContextException
   */
  public function moveBlock(Request $request, SectionStorageInterface $section_storage, string $nested_storage_path = NULL) {
    $delta_from = $request->get('delta_from');
    $delta_to = $request->get('delta_to');
    $region_to = $request->get('region_to');
    $block_uuid = $request->get('block_uuid');
    $preceding_block_uuid = $request->get('preceding_block_uuid');
    $nested_storage_path_from = $request->get('nested_storage_path_from');
    $nested_storage_path_to = $request->get('nested_storage_path_to');
    // Get the section storage where the block will be placed.
    $section_storage_to = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path_to);

    // Remove the component from the section storage.
    if ($nested_storage_path_to !== $nested_storage_path_from) {
      $from_section_storage = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path_from);
      $section = $from_section_storage->getSection($delta_from);
    } else {
      $section = $section_storage_to->getSection($delta_from);
    }
    $component = $section->getComponent($block_uuid);
    $section->removeComponent($block_uuid);

    // If the block is moving to a new section storage update the from section
    // storage before placing the block.
    if ($nested_storage_path_to !== $nested_storage_path_from) {
      $section_storage = $this->sectionStorageHandler->updateSectionStorage($section_storage, $nested_storage_path_from, $from_section_storage);
      $section_storage_to = $this->sectionStorageHandler->getCurrentSectionStorage($section_storage, $nested_storage_path_to);
    }

    // Get the destination section.
    $section = $section_storage_to->getSection($delta_to);

    // If a preceding block was specified, insert after that. Otherwise add the
    // block to the front.
    $component->setRegion($region_to);
    if (isset($preceding_block_uuid)) {
      $section->insertAfterComponent($preceding_block_uuid, $component);
    }
    else {
      $section->insertComponent(0, $component);
    }
    $section_storage = $this->sectionStorageHandler->updateSectionStorage($section_storage, $nested_storage_path_to, $section_storage_to);

    // Prepare an updated Layout Builder UI response.
    $response = new AjaxResponse();
    $response->addCommand(new RemoveCommand('[data-drupal-messages]'));
    $layout = [
      '#type' => 'layout_builder_plus',
      '#section_storage' => $section_storage,
      '#nested_storage_path' => $nested_storage_path_to,
    ];
    $selector = '#layout-builder';
    if (!empty($nested_storage_path_to)) {
      $pieces = SectionStorageHandler::decodeNestedStoragePath($nested_storage_path_to);
      $storage_uuid = end($pieces);
      $selector = "[data-nested-storage-uuid='$storage_uuid']";
    }
    $response->addCommand(new ReplaceCommand($selector, $layout));
    return $response;
  }

}
